SystemOrganization addCategory: #'Pier-Security-Model'!
SystemOrganization addCategory: #'Pier-Security-Command'!
SystemOrganization addCategory: #'Pier-Security-Widgets'!
SystemOrganization addCategory: #'Pier-Security-Tests'!

!PRStructure class methodsFor: '*pier-security-descriptions' stamp: 'lr 3/26/2009 20:58'!
descriptionGroup
	^ MAToOneRelationDescription new
		classes: (Array with: PUGroup);
		parameterName: 'group';
		accessor: #group;
		label: 'Group';
		priority: 210;
		yourself! !

!PRStructure class methodsFor: '*pier-security-descriptions' stamp: 'lr 3/26/2009 20:58'!
descriptionOwner
	^ MAToOneRelationDescription new
		classes: (Array with: PUUser);
		parameterName: 'owner';
		accessor: #owner;
		label: 'Owner';
		priority: 200;
		yourself! !

!PRStructure methodsFor: '*pier-security-accessing' stamp: 'lr 3/9/2006 11:30'!
group
	"Answer the owning-group of the file."

	^ self securityDecoration ifNotNilDo: [ :value | value group ]! !

!PRStructure methodsFor: '*pier-security-accessing' stamp: 'lr 3/9/2006 11:30'!
owner
	"Answer the owning-user of the receiver."

	^ self securityDecoration ifNotNilDo: [ :value | value owner ]! !

!PRStructure methodsFor: '*pier-security-decorations' stamp: 'lr 3/9/2006 11:30'!
securityDecoration
	^ self
		decorationOfClass: self securityDecorationClass
		ifAbsent: [ self securityDecorationClass onStructureAdded: self ]! !

!PRStructure methodsFor: '*pier-security-decorations' stamp: 'lr 3/9/2006 11:30'!
securityDecorationClass
	^ PUSecurity! !

PRViewComponent subclass: #PUBrowseComponent
	instanceVariableNames: 'permissions'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pier-Security-Widgets'!

!PUBrowseComponent class methodsFor: 'testing' stamp: 'lr 10/17/2006 07:15'!
isAbstract
	^ false! !

!PUBrowseComponent class methodsFor: 'accessing' stamp: 'lr 10/17/2006 07:18'!
label
	^ 'Browse'! !

!PUBrowseComponent methodsFor: 'configuration' stamp: 'lr 2/12/2007 19:07'!
defaultPermissions
	^ (Array streamContents: [ :stream |
		PRCommand allSubclasses do: [ :each |
			each isAbstract
				ifFalse: [ stream nextPutAll: each permissions ] ] ])
		asSortedCollection: [ :a :b | a label < b label ]! !

!PUBrowseComponent methodsFor: 'actions' stamp: 'lr 1/29/2008 09:02'!
execute: aClass on: aStructure
	| command |
	command := aClass new.
	(self context structure: aStructure command: command) isValid
		ifFalse: [ ^ self inform: 'You are not authorized to execute: ' , command label , '.' ].
	command isConfirmed ifTrue: [
		(self confirm: 'Are you sure to perform the selected action: ' , command label , '?')
			ifFalse: [ ^ self ] ].
	command isQuick
		ifFalse: [ command := self call: command asComponent ].
	command ifNotNil: [ command execute ]! !

!PUBrowseComponent methodsFor: 'initialization' stamp: 'lr 10/17/2006 07:19'!
initialize
	super initialize.
	permissions := self defaultPermissions! !

!PUBrowseComponent methodsFor: 'accessing' stamp: 'lr 10/17/2006 07:20'!
permissions
	^ permissions! !

!PUBrowseComponent methodsFor: 'rendering' stamp: 'lr 10/17/2006 20:30'!
renderContentOn: html
	self renderPathOn: html.
	self renderTableOn: html! !

!PUBrowseComponent methodsFor: 'rendering' stamp: 'lr 7/13/2007 18:13'!
renderPathOn: html
	html div: [
		self structure parents do: [ :each |
			html text: '/ '.
			html anchor
				attributeAt: 'title' put: each title;
				goto: (self context structure: each view: self class);
				with: each name.
			html text: ' ' ] ].
	html break! !

!PUBrowseComponent methodsFor: 'rendering' stamp: 'lr 2/12/2007 19:07'!
renderPermissions: aStructure permissions: aCollection on: html
	| current short |
	current := self context structure: aStructure.
	self permissions do: [ :each |
		html code
			attributeAt: 'title' put: each label;
			with: [
				short := (aCollection includes: each)
					ifTrue: [ each label first asLowercase ]
					ifFalse: [ $- ].
				(each command isValidIn: current)
					ifFalse: [ html render: short ]
					ifTrue: [
						html anchor
							goto: (self context structure: aStructure command: each command new);
							with: short ] ] ]! !

!PUBrowseComponent methodsFor: 'rendering' stamp: 'lr 3/18/2008 21:49'!
renderTableHeadOn: html
	html tableRow: [
		html tableHeading: 'name'.
		html tableHeading: 'type'.
		html tableHeading: 'owner mode'.
		html tableHeading: 'group mode'.
		html tableHeading: 'other mode'.
		html tableHeading: 'owner'.
		html tableHeading: 'group' ]! !

!PUBrowseComponent methodsFor: 'rendering' stamp: 'lr 9/27/2007 10:05'!
renderTableOn: html
	html table: [
		html tableHead: [
			self renderTableHeadOn: html ].
		html tableBody: [
			self structure parent
				ifNotNilDo: [ :parent | self renderTableRow: parent named: '..' on: html ].
			self renderTableRow: self structure named: '.' on: html.
			(self context enumerator
				select: [ :each | each isValidCommand: each viewCommandClass in: self context ])
				do: [ :each | self renderTableRow: each named: each name on: html ] ] ]! !

!PUBrowseComponent methodsFor: 'rendering' stamp: 'lr 3/18/2008 21:40'!
renderTableRow: aStructure named: aString on: html
	html tableRow attributeAt: 'title' put: aStructure title; with: [
		html tableData: [
			html anchor
				goto: (self context structure: aStructure view: self class);
				with: aString ].
		html tableData: aStructure class label.
		html tableData: [
			self
				renderPermissions: aStructure
				permissions: aStructure securityDecoration ownerPermissions
				on: html ].
		html tableData: [
			self
				renderPermissions: aStructure
				permissions: aStructure securityDecoration groupPermissions
				on: html ].
		html tableData: [
			self
				renderPermissions: aStructure
				permissions: aStructure securityDecoration otherPermissions
				on: html ].
		html tableData: (aStructure securityDecoration owner
			ifNotNilDo: [ :each | each name ]).
		html tableData: (aStructure securityDecoration group
			ifNotNilDo: [ :each | each name ]) ]! !

!PUBrowseComponent methodsFor: 'accessing' stamp: 'lr 10/17/2006 07:21'!
structure
	^ self context structure! !

TestCase subclass: #PUMemberTest
	instanceVariableNames: 'group1 group2 user1 user2 user3'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pier-Security-Tests'!

!PUMemberTest methodsFor: 'running' stamp: 'lr 9/27/2006 11:47'!
setUp
	group1 := PUGroup named: 'group 1'.
	group2 := PUGroup named: 'group 2'.

	user1 := PUUser named: 'user 1'.
	user2 := PUUser named: 'user 2'.
	user3 := PUUser named: 'user 3'! !

!PUMemberTest methodsFor: 'testing' stamp: 'lr 2/12/2007 19:07'!
testEqualitiy
	self assert: group1 = group1.
	self assert: group2 = group2.

	self assert: user1 = user1.
	self assert: user2 = user2.
	self assert: user3 = user3.

	self deny: group1 = group2.
	self deny: group2 = group1.

	self deny: user1 = user2.
	self deny: user2 = user3.
	self deny: user3 = user1.

	self deny: group1 = user1.
	self deny: group2 = user1.
	self deny: user1 = group1.
	self deny: user2 = group1.
	self deny: user3 = group1! !

!PUMemberTest methodsFor: 'testing-groups' stamp: 'lr 6/22/2008 18:31'!
testGroupUsers
	group1 users: (Array with: user1 with: user2).
	self assert: group1 users equalElements: (Set with: user1 with: user2).
	self assert: user1 groups = (OrderedCollection with: group1).
	self assert: user2 groups = (OrderedCollection with: group1).
	self assert: user3 groups = (OrderedCollection new).

	group1 users: (Array with: user2 with: user3).
	self assert: group1 users equalElements: (Set with: user2 with: user3).
	self assert: user1 groups = (OrderedCollection new).
	self assert: user2 groups = (OrderedCollection with: group1).
	self assert: user3 groups = (OrderedCollection with: group1)! !

!PUMemberTest methodsFor: 'testing' stamp: 'lr 9/27/2006 11:49'!
testName
	self assert: group1 name = 'group 1'.
	self assert: group2 name = 'group 2'.

	self assert: user1 name = 'user 1'.
	self assert: user2 name = 'user 2'.
	self assert: user3 name = 'user 3'! !

!PUMemberTest methodsFor: 'testing-user' stamp: 'lr 6/22/2008 18:31'!
testUserAddGroups
	user1 addGroup: group1; addGroup: group2.
	self assert: user1 group = group1.
	self assert: user1 groups = (OrderedCollection with: group1 with: group2).
	self assert: group1 users equalElements: (Set with: user1).
	self assert: group2 users equalElements: (Set with: user1)! !

!PUMemberTest methodsFor: 'testing-user' stamp: 'lr 6/22/2008 18:31'!
testUserGroups
	user1 groups: (Array with: group1 with: group2).
	self assert: user1 groups = (OrderedCollection with: group1 with: group2).
	self assert: group1 users equalElements: (Set with: user1).
	self assert: group2 users equalElements: (Set with: user1).

	user1 groups: (Array with: group1).
	self assert: user1 groups = (OrderedCollection with: group1).
	self assert: group1 users equalElements: (Set with: user1).
	self assert: group2 users equalElements: (Set new)! !

!PUMemberTest methodsFor: 'testing-user' stamp: 'lr 9/11/2008 16:19'!
testUserPassword
	user1 password: 'foo'.
	self deny: user1 password = 'foo'.
	self assert: (user1 validatePassword: 'foo').
	
	user1 password: user1 password.
	self assert: (user1 validatePassword: 'foo')! !

!PUMemberTest methodsFor: 'testing-user' stamp: 'lr 6/22/2008 18:31'!
testUserRemoveGroups
	user1 addGroup: group1; addGroup: group2; removeGroup: group2.
	self assert: user1 groups = (OrderedCollection with: group1).
	self assert: group1 users equalElements: (Set with: user1).
	self assert: group2 users equalElements: (Set new)! !

!PUMemberTest methodsFor: 'testing-user' stamp: 'lr 9/27/2006 11:53'!
testUserSuperuser
	self deny: user1 superuser.
	self deny: user1 isSuperuser.

	user1 superuser: true.
	self assert: user1 superuser.
	self assert: user1 isSuperuser! !

!TestCase methodsFor: '*pier-security' stamp: 'lr 6/22/2008 18:28'!
assert: aCollection1 equalElements: aCollection2
	self assert: aCollection1 size = aCollection2 size.
	aCollection1 do: [ :each | self assert: (aCollection2 includes: each) ]! !

!PRVisitor methodsFor: '*pier-security-visiting-structure' stamp: 'lr 10/16/2007 22:28'!
visitSecurity: anObject! !

!PRContext class methodsFor: '*pier-security' stamp: 'lr 3/26/2009 20:58'!
descriptionUser
	^ MASingleOptionDescription new
		accessor: (MASelectorAccessor read: #user write: #setUser:);
		reference: PUUser description;
		parameterName: 'user';
		label: 'User';
		yourself! !

!PRContext methodsFor: '*pier-security-accessing' stamp: 'lr 3/9/2006 11:30'!
group
	^ self user group! !

!PRContext methodsFor: '*pier-security-testing' stamp: 'lr 3/9/2006 11:30'!
isLoggedIn
	^ self user notNil! !

!PRContext methodsFor: '*pier-security-initialization' stamp: 'lr 9/27/2007 10:03'!
setUser: aUser
	self propertyAt: #user put: aUser! !

!PRContext methodsFor: '*pier-security-accessing' stamp: 'lr 3/9/2006 11:30'!
user
	^ self propertyAt: #user ifAbsentPut: [ nil ]! !

!PRContext methodsFor: '*pier-security-navigation' stamp: 'lr 3/9/2006 11:30'!
user: anUser
	"Create a copy of the current context with the current user replaced by anUser."

	^ self copy
		setUser: anUser;
		yourself! !

!PRAddCommandTest methodsFor: '*pier-security-testing' stamp: 'lr 6/22/2008 18:29'!
testGroup
	| parentDecoration childDecoration |
	parentDecoration := self context structure
		securityDecoration.
	parentDecoration
		owner: self bloggerUser;
		ownerPermissions: PUChangeSecurity permissionInstances;
		group: self adminGroup;
		groupPermissions: PRAddCommand permissions;
		otherPermissions: #().
	self context
		setUser: self bloggerUser.
	PRCurrentContext
		use: self
		during: [ self command execute ].

	childDecoration := self command answer structure
		securityDecoration.
	self assert: childDecoration owner = self bloggerUser.
	self assert: childDecoration ownerPermissions equalElements: PUChangeSecurity permissionInstances.
	self assert: childDecoration group = self adminGroup.
	self assert: childDecoration groupPermissions equalElements: PRAddCommand permissions.
	self assert: childDecoration otherPermissions equalElements: #()! !

!PRAddCommandTest methodsFor: '*pier-security-testing' stamp: 'lr 6/22/2008 18:29'!
testOwner
	| parentDecoration childDecoration |
	parentDecoration := self context structure
		securityDecoration.
	parentDecoration
		owner: self adminUser;
		ownerPermissions: PRAddCommand permissions;
		group: self bloggerGroup;
		groupPermissions: PUChangeSecurity permissionInstances;
		otherPermissions: #().
	self context
		setUser: self bloggerUser.
	PRCurrentContext
		use: self
		during: [ self command execute ].

	childDecoration := self command answer structure
		securityDecoration.
	self assert: childDecoration owner = self bloggerUser.
	self assert: childDecoration ownerPermissions equalElements: PRAddCommand permissions.
	self assert: childDecoration group = self bloggerGroup.
	self assert: childDecoration groupPermissions equalElements: PUChangeSecurity permissionInstances.
	self assert: childDecoration otherPermissions equalElements: #()! !

!PRAddCommandTest methodsFor: '*pier-security-testing' stamp: 'lr 6/22/2008 18:30'!
testWorld
	| parentDecoration childDecoration |
	parentDecoration := self context structure
		securityDecoration.
	parentDecoration
		owner: self adminUser;
		ownerPermissions: PUChangeSecurity permissionInstances;
		group: self adminGroup;
		groupPermissions: PUChangeSecurity permissionInstances;
		otherPermissions: PRAddCommand permissions.
	self context
		setUser: nil.
	PRCurrentContext
		use: self
		during: [ self command execute ].

	childDecoration := self command answer structure
		securityDecoration.
	self assert: childDecoration owner = self adminUser.
	self assert: childDecoration ownerPermissions equalElements: PUChangeSecurity permissionInstances.
	self assert: childDecoration group = self adminGroup.
	self assert: childDecoration groupPermissions equalElements: PUChangeSecurity permissionInstances.
	self assert: childDecoration otherPermissions equalElements: PRAddCommand permissions! !

!PRKernel methodsFor: '*pier-security-defaults' stamp: 'lr 9/27/2006 11:45'!
adminGroup
	^ self propertyAt: #adminGroup ifAbsentPut: [ PUGroup named: 'admin' ]! !

!PRKernel methodsFor: '*pier-security-defaults' stamp: 'lr 9/27/2006 11:45'!
adminUser
	^ self propertyAt: #adminUser ifAbsentPut: [
		(PUUser named: 'admin')
			addGroup: self adminGroup;
			password: 'pier';
			superuser: true;
			yourself ]! !

!PRKernel methodsFor: '*pier-security-accessing' stamp: 'lr 2/5/2009 21:34'!
groupNamed: aString ifNone: aBlock
	^ self groups 
		detect: [ :each | each name = aString ]
		ifNone: aBlock! !

!PRKernel methodsFor: '*pier-security-accessing' stamp: 'lr 3/9/2006 11:30'!
groups
	^ self propertyAt: #groups ifAbsentPut: [ Set with: self adminGroup ]! !

!PRKernel methodsFor: '*pier-security-accessing' stamp: 'lr 2/5/2009 21:58'!
userNamed: aString ifNone: aBlock
	^ self users
		detect: [ :each | each name = aString ]
		ifNone: aBlock! !

!PRKernel methodsFor: '*pier-security-accessing' stamp: 'lr 3/9/2006 11:30'!
users
	^ self propertyAt: #users ifAbsentPut: [ Set with: self adminUser ]! !

PRDecoration subclass: #PUSecurity
	instanceVariableNames: 'owner ownerPermissions group groupPermissions otherPermissions'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pier-Security-Model'!

!PUSecurity commentStamp: 'lr 12/20/2005 15:05' prior: 0!
Every structure is owned by a specific user. The owner of an object comprises its user class. Permissions assigned to the user class only apply to that specific user. A structure is also assigned a group, which comprises its group class. Permissions assigned to the group class only apply to members of that group. Users who are not otherwise represented by the other two classes comprise a file's others class.

The effective permissions that have applied to a specific user in relation to a file are determined in logical precedence. For example, the user who owns the file will have the effective permissions given to the user class regardless of those assigned to the group or others class.!

!PUSecurity class methodsFor: 'testing' stamp: 'lr 3/9/2006 11:31'!
isAbstract
	^ false! !

!PUSecurity class methodsFor: 'events' stamp: 'lr 4/5/2007 17:59'!
onStructureAdded: aStructure
	| decoration |
	decoration := aStructure isRoot
		ifTrue: [ aStructure securityDecorationClass new ]
		ifFalse: [ aStructure parent securityDecoration copy ].
	PRCurrentContext value ifNotNilDo: [ :context |
		context user ifNotNilDo: [ :user |
			decoration owner: user.
			decoration group
				ifNil: [ decoration group: user group ] ] ].
	^ aStructure
		addDecoration: decoration
		ifPresent: [ :value | value ]! !

!PUSecurity methodsFor: 'visiting' stamp: 'lr 3/9/2006 11:31'!
accept: aVisitor
	aVisitor visitSecurity: self! !

!PUSecurity methodsFor: 'testing' stamp: 'lr 9/11/2006 11:02'!
allows: aCommandClass permissions: aCollection
	^ aCollection anySatisfy: [ :each | each command = aCommandClass ]! !

!PUSecurity methodsFor: 'accessing' stamp: 'lr 3/9/2006 11:31'!
group
	^ group! !

!PUSecurity methodsFor: 'accessing' stamp: 'lr 3/9/2006 11:31'!
group: aGroup
	group := aGroup! !

!PUSecurity methodsFor: 'accessing-permissions' stamp: 'lr 9/11/2006 10:28'!
groupPermissions
	^ groupPermissions ifNil: [ groupPermissions := self ownerPermissions copy ]! !

!PUSecurity methodsFor: 'accessing-permissions' stamp: 'lr 9/11/2006 10:28'!
groupPermissions: aSet
	groupPermissions := aSet asSet! !

!PUSecurity methodsFor: 'initialization' stamp: 'lr 9/11/2006 10:48'!
initialize
	super initialize.
	self ownerPermissions; groupPermissions; otherPermissions! !

!PUSecurity methodsFor: 'testing' stamp: 'lr 9/11/2006 11:02'!
isAllowedCommand: aCommandClass in: aContext
	(aContext user notNil and: [ aContext user isSuperuser ])
		ifTrue: [ ^ true ].
	(self owner = aContext user)
		ifTrue: [ ^ self allows: aCommandClass permissions: self ownerPermissions ].
	(self group notNil and: [ self group includes: aContext user ])
		ifTrue: [ ^ self allows: aCommandClass permissions: self groupPermissions ].
	^ self allows: aCommandClass permissions: self otherPermissions! !

!PUSecurity methodsFor: 'accessing-permissions' stamp: 'lr 9/11/2006 10:29'!
otherPermissions
	^ otherPermissions ifNil: [ otherPermissions := Set withAll: PRViewCommand permissions ]! !

!PUSecurity methodsFor: 'accessing-permissions' stamp: 'lr 9/11/2006 10:29'!
otherPermissions: aSet
	otherPermissions := aSet asSet! !

!PUSecurity methodsFor: 'accessing' stamp: 'lr 3/9/2006 11:31'!
owner
	^ owner! !

!PUSecurity methodsFor: 'accessing' stamp: 'lr 3/9/2006 11:31'!
owner: anUser
	owner := anUser! !

!PUSecurity methodsFor: 'accessing-permissions' stamp: 'lr 9/11/2006 10:29'!
ownerPermissions
	^ ownerPermissions ifNil: [ ownerPermissions := PUChangeSecurity permissionInstances asSet ]! !

!PUSecurity methodsFor: 'accessing-permissions' stamp: 'lr 9/11/2006 10:30'!
ownerPermissions: aSet
	ownerPermissions := aSet asSet! !

!PUSecurity methodsFor: 'copying' stamp: 'lr 9/11/2006 10:30'!
postCopy
	super postCopy.
	self ownerPermissions: self ownerPermissions copy.
	self groupPermissions: self groupPermissions copy.
	self otherPermissions: self otherPermissions copy! !

!PUSecurity methodsFor: 'configuration' stamp: 'lr 3/9/2006 11:31'!
priority
	^ -200! !

!PUSecurity methodsFor: 'testing' stamp: 'lr 2/12/2007 19:07'!
validateCommand: aCommand
	(aCommand context user notNil
		and: [ aCommand context user isSuperuser ])
			ifTrue: [ ^ self ].
	(self owner = aCommand context user
		and: [ self ownerPermissions includes: aCommand permission ])
			ifTrue: [ ^ self ].
	(self group notNil
		and: [ (self group includes: aCommand context user)
		and: [ self groupPermissions includes: aCommand permission ] ])
			ifTrue: [ ^ self ].
	(self otherPermissions includes: aCommand permission)
		ifTrue: [ ^ self ].
	MAConditionError description: self description signal: 'Insufficient permissions'! !

MAValidationError subclass: #PUInvalidLogin
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pier-Security-Model'!

!PRCommandTest methodsFor: '*pier-security-accessing' stamp: 'lr 3/9/2006 11:30'!
adminGroup
	^ self kernel adminGroup! !

!PRCommandTest methodsFor: '*pier-security-accessing' stamp: 'lr 3/9/2006 11:30'!
adminUser
	^ self kernel adminUser! !

!PRCommandTest methodsFor: '*pier-security-accessing' stamp: 'mberth 8/21/2007 16:41'!
bloggerGroup
	^ self kernel groupNamed: 'blogger'
		ifNone: [self kernel groups add: self bloggerGroupInstance]! !

!PRCommandTest methodsFor: '*pier-security-private' stamp: 'lr 3/9/2006 11:30'!
bloggerGroupInstance
	^ PUGroup named: 'blogger'! !

!PRCommandTest methodsFor: '*pier-security-accessing' stamp: 'mberth 8/21/2007 16:42'!
bloggerUser
	^ self kernel userNamed: 'james'
		ifNone: [self kernel users add: self bloggerUserInstance]! !

!PRCommandTest methodsFor: '*pier-security-private' stamp: 'lr 3/9/2006 11:30'!
bloggerUserInstance
	^ (PUUser named: 'james')
		addGroup: self bloggerGroup;
		password: 'bottom';
		yourself! !

PRCommandTest subclass: #PUChangeSecurityTest
	instanceVariableNames: 'child'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pier-Security-Tests'!

PUChangeSecurityTest subclass: #PUChangeGroupTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pier-Security-Tests'!

!PUChangeGroupTest class methodsFor: 'testing' stamp: 'lr 3/9/2006 11:31'!
isAbstract
	^ false! !

!PUChangeGroupTest methodsFor: 'private' stamp: 'lr 3/9/2006 11:30'!
actualClass
	^ PUChangeGroup! !

!PUChangeGroupTest methodsFor: 'testing' stamp: 'lr 9/27/2007 10:08'!
testGroup
	self command
		with: true; all: true;
		group: self bloggerGroup.
	self command execute.

	self assert: self structure group = self bloggerGroup.
	self assert: self child group = self bloggerGroup! !

PUChangeSecurityTest subclass: #PUChangeOtherTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pier-Security-Tests'!

!PUChangeOtherTest class methodsFor: 'testing' stamp: 'lr 3/9/2006 11:31'!
isAbstract
	^ false! !

!PUChangeOtherTest methodsFor: 'private' stamp: 'lr 3/9/2006 11:30'!
actualClass
	^ PUChangeOther! !

PUChangeSecurityTest subclass: #PUChangeOwnerTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pier-Security-Tests'!

!PUChangeOwnerTest class methodsFor: 'testing' stamp: 'lr 3/9/2006 11:31'!
isAbstract
	^ false! !

!PUChangeOwnerTest methodsFor: 'private' stamp: 'lr 3/9/2006 11:30'!
actualClass
	^ PUChangeOwner! !

!PUChangeOwnerTest methodsFor: 'testing' stamp: 'lr 9/27/2007 10:08'!
testOwner
	self command
		with: true; all: true;
		owner: self bloggerUser.
	self command execute.

	self assert: self structure owner = self bloggerUser.
	self assert: self child owner = self bloggerUser! !

!PUChangeSecurityTest methodsFor: 'private' stamp: 'lr 3/9/2006 11:30'!
actualClass
	^ PUChangeSecurity! !

!PUChangeSecurityTest methodsFor: 'accessing' stamp: 'lr 3/9/2006 11:30'!
child
	^ child ifNil: [ child := self childInstance ]! !

!PUChangeSecurityTest methodsFor: 'private' stamp: 'lr 3/9/2006 11:30'!
childInstance
	^ PRPage named: 'child'! !

!PUChangeSecurityTest methodsFor: 'accessing' stamp: 'lr 9/11/2006 10:42'!
permissions
	^ Set new
		addAll: PRAddCommand permissions;
		addAll: PRRemoveCommand permissions;
		yourself! !

!PUChangeSecurityTest methodsFor: 'private' stamp: 'lr 9/27/2007 10:08'!
structureInstance
	^ super structureInstance
		addChild: self child;
		yourself! !

!PUChangeSecurityTest methodsFor: 'testing-operations' stamp: 'lr 9/27/2007 10:07'!
testOperationAdd
	self command
		with: true; all: true;
		operator: #add; permissions: self permissions.
	self command execute.

	self assert: ((self command accessor read: self structure)
		includesAllOf: self permissions).
	self assert: ((self command accessor read: self child)
		includesAllOf: self permissions)! !

!PUChangeSecurityTest methodsFor: 'testing-operations' stamp: 'lr 9/27/2007 10:07'!
testOperationRemove
	self command
		with: true; all: true;
		operator: #remove; permissions: self permissions.
	self command execute.

	self deny: ((self command accessor read: self structure)
		includesAnyOf: self permissions).
	self deny: ((self command accessor read: self child)
		includesAnyOf: self permissions)! !

!PUChangeSecurityTest methodsFor: 'testing-operations' stamp: 'lr 6/22/2008 18:30'!
testOperationSet
	self command
		with: true; all: true;
		operator: #set; permissions: self permissions.
	self command execute.

	self assert: (self command accessor read: self structure) equalElements: self permissions.
	self assert: (self command accessor read: self child) equalElements: self permissions! !

!PUChangeSecurityTest methodsFor: 'testing-operations' stamp: 'lr 6/22/2008 18:30'!
testOperationUnchanged
	| structurePermission childPermissions |
	structurePermission := (self command accessor read: self structure)
		copy.
	childPermissions := (self command accessor read: self child)
		copy.

	self command
		with: true; all: true;
		operator: nil; permissions: self permissions.
	self command execute.

	self assert: (self command accessor
		read: self structure) equalElements: structurePermission.
	self assert: (self command accessor
		read: self child) equalElements: childPermissions! !

PRCommandTest subclass: #PULoginTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pier-Security-Tests'!

!PULoginTest class methodsFor: 'testing' stamp: 'lr 3/9/2006 11:31'!
isAbstract
	^ false! !

!PULoginTest methodsFor: 'private' stamp: 'lr 3/9/2006 11:30'!
actualClass
	^ PULogin! !

!PULoginTest methodsFor: 'testing-testing' stamp: 'lr 3/9/2006 11:30'!
testIsLogged
	self deny: self command isLogged! !

!PULoginTest methodsFor: 'testing' stamp: 'lr 1/12/2007 15:37'!
testLoginAdmin
	self assert: self context user isNil.
	self command
		username: self adminUser name;
		password: 'pier'.
	self
		shouldnt: [ self command execute ]
		raise: PUInvalidLogin.
	self assert: self command user = self adminUser.
	self assert: self command answer user = self adminUser.
	self assert: self command answer command class = PRViewCommand.
	self assert: self context user isNil! !

!PULoginTest methodsFor: 'testing' stamp: 'lr 1/12/2007 15:37'!
testLoginBlogger
	self assert: self context user isNil.
	self command
		username: self bloggerUser name;
		password: 'bottom'.
	self
		shouldnt: [ self command execute ]
		raise: PUInvalidLogin.
	self assert: self command user = self bloggerUser.
	self assert: self command answer user = self bloggerUser.
	self assert: self command answer command class = PRViewCommand.
	self assert: self context user isNil! !

!PULoginTest methodsFor: 'testing' stamp: 'lr 1/12/2007 15:37'!
testLoginInvalidPassword
	self assert: self context user isNil.
	self command username: self bloggerUser name.
	self should: [ self command execute ] raise: PUInvalidLogin.
	self assert: self context user isNil! !

!PULoginTest methodsFor: 'testing' stamp: 'lr 1/12/2007 15:37'!
testLoginInvalidUsername
	self assert: self context user isNil.
	self should: [ self command execute ] raise: PUInvalidLogin.
	self assert: self context user isNil! !

PRCommandTest subclass: #PULogoutTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pier-Security-Tests'!

!PULogoutTest class methodsFor: 'testing' stamp: 'lr 3/9/2006 11:31'!
isAbstract
	^ false! !

!PULogoutTest methodsFor: 'private' stamp: 'lr 3/9/2006 11:31'!
actualClass
	^ PULogout! !

!PULogoutTest methodsFor: 'private' stamp: 'lr 3/9/2006 11:31'!
contextInstance
	^ super contextInstance user: self bloggerUser! !

!PULogoutTest methodsFor: 'testing-testing' stamp: 'lr 3/9/2006 11:31'!
testIsLogged
	self deny: self command isLogged! !

!PULogoutTest methodsFor: 'testing-testing' stamp: 'lr 3/9/2006 11:31'!
testIsQuick
	self assert: self command isQuick! !

!PULogoutTest methodsFor: 'testing' stamp: 'lr 1/12/2007 15:36'!
testingLogout
	self assert: self context user = self bloggerUser.
	self command execute.
	self assert: self command answer user isNil.
	self assert: self command answer command class = PRViewCommand.
	self assert: self context user = self bloggerUser! !

PRObject subclass: #PUMember
	instanceVariableNames: 'name'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pier-Security-Model'!

PUMember subclass: #PUGroup
	instanceVariableNames: 'users'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pier-Security-Model'!

!PUGroup class methodsFor: 'accessing-defaults' stamp: 'lr 9/27/2006 11:22'!
defaultUsers
	^ PRCurrentContext value kernel users! !

!PUGroup class methodsFor: 'accessing-description' stamp: 'lr 4/10/2008 14:17'!
descriptionUsers
	^ MAMultipleOptionDescription new
		accessor: #users;
		label: 'Users';
		priority: 200;
		options: [ self defaultUsers ] asDynamicObject;
		componentClass: MAListCompositonComponent;
		reference: (PUUser descriptionContainer
				add: PUUser descriptionName;
				yourself);
		beDistinct;
		yourself! !

!PUGroup class methodsFor: 'testing' stamp: 'lr 3/9/2006 11:31'!
isAbstract
	^ false! !

!PUGroup methodsFor: 'testing' stamp: 'lr 3/9/2006 11:30'!
includes: anUser
	^ self users includes: anUser! !

!PUGroup methodsFor: 'initialization' stamp: 'lr 9/27/2006 12:02'!
initialize
	super initialize.
	users := Set new! !

!PUGroup methodsFor: 'accessing' stamp: 'lr 3/9/2006 11:30'!
users
	^ users! !

!PUGroup methodsFor: 'accessing' stamp: 'lr 2/12/2007 19:07'!
users: aCollection
	(aCollection copyWithoutAll: self users) do: [ :user |
		(user groups includes: self)
			ifFalse: [ user groups add: self ] ].
	(self users copyWithoutAll: aCollection) do: [ :user |
		(user groups includes: self)
			ifTrue: [ user groups remove: self ] ].
	users := aCollection asSet! !

!PUMember class methodsFor: 'accessing-description' stamp: 'lr 4/10/2008 14:17'!
descriptionName
	^ MAStringDescription new
		accessor: #name;
		label: 'Name';
		priority: 100;
		propertyAt: #public
			put: true;
		beReadonly;
		beRequired;
		yourself! !

!PUMember class methodsFor: 'instance-creation' stamp: 'lr 3/9/2006 11:31'!
named: aString
	^ self new
		setName: aString;
		yourself! !

!PUMember methodsFor: 'comparing' stamp: 'lr 3/9/2006 11:31'!
= anObject
	^ self class = anObject class
		and: [ self name = anObject name ]! !

!PUMember methodsFor: 'comparing' stamp: 'lr 3/9/2006 11:31'!
hash
	^ self class hash bitXor: self name hash! !

!PUMember methodsFor: 'accessing' stamp: 'lr 3/9/2006 11:31'!
name
	^ name! !

!PUMember methodsFor: 'printing' stamp: 'lr 3/9/2006 11:31'!
printOn: aStream
	super printOn: aStream.
	aStream nextPutAll: ' name: '; print: self name! !

!PUMember methodsFor: 'initialize-release' stamp: 'lr 3/9/2006 11:31'!
setName: aString
	name := aString! !

PUMember subclass: #PUUser
	instanceVariableNames: 'password groups superuser'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pier-Security-Model'!

!PUUser class methodsFor: 'accessing-defaults' stamp: 'lr 9/27/2006 10:47'!
defaultGroups
	^ PRCurrentContext value kernel groups! !

!PUUser class methodsFor: 'accessing-description' stamp: 'lr 2/5/2009 21:09'!
descriptionGroups
	^ MAMultipleOptionDescription new
		addCondition: (MACondition selector: #notEmpty)
			labelled: 'A user has to belong to at least one group';
		options: [ self defaultGroups ] asDynamicObject;
		componentClass: MAListCompositonComponent;
		reference: (PUGroup descriptionContainer
			add: PUGroup descriptionName;
			yourself);
		accessor: #groups;
		label: 'Groups';
		priority: 300;
		beDistinct;
		beOrdered;
		yourself! !

!PUUser class methodsFor: 'accessing-description' stamp: 'lr 4/10/2008 14:17'!
descriptionPassword
	^ MAPasswordDescription new
		accessor: #password;
		label: 'Password';
		priority: 200;
		propertyAt: #public
			put: true;
		beRequired;
		yourself! !

!PUUser class methodsFor: 'accessing-description' stamp: 'lr 4/10/2008 14:17'!
descriptionSuperuser
	^ MABooleanDescription new
		comment: 'Superusers have unrestricted rights, they can also edit users and groups.';
		accessor: #superuser;
		label: 'Superuser';
		priority: 400;
		yourself! !

!PUUser class methodsFor: 'testing' stamp: 'lr 3/9/2006 11:31'!
isAbstract
	^ false! !

!PUUser methodsFor: 'utilties' stamp: 'lr 9/27/2006 11:54'!
addGroup: aGroup
	(self groups includes: aGroup)
		ifFalse: [ self groups add: aGroup ].
	aGroup users add: self! !

!PUUser methodsFor: 'accessing-readonly' stamp: 'lr 3/9/2006 11:31'!
group
	^ self groups first! !

!PUUser methodsFor: 'accessing-readonly' stamp: 'lr 3/9/2006 11:31'!
groups
	^ groups! !

!PUUser methodsFor: 'accessing-readonly' stamp: 'lr 9/27/2006 12:03'!
groups: aCollection
	self groups do: [ :each | each users remove: self ifAbsent: [ ] ].
	aCollection do: [ :each | each users add: self ].
	groups := aCollection asOrderedCollection! !

!PUUser methodsFor: 'initialization' stamp: 'lr 3/9/2006 11:31'!
initialize
	super initialize.
	groups := OrderedCollection new.
	superuser := false! !

!PUUser methodsFor: 'testing' stamp: 'lr 3/9/2006 11:31'!
isSuperuser
	^ self superuser! !

!PUUser methodsFor: 'accessing' stamp: 'lr 9/11/2008 16:17'!
password
	^ password isNil ifFalse: [ String new: 8 withAll: $* ]! !

!PUUser methodsFor: 'accessing' stamp: 'lr 9/11/2008 16:18'!
password: aString
	(aString isNil or: [ aString isEmpty or: [ self password = aString ] ])
		ifTrue: [ ^ self ].
	password := PRCompatibility hashPassword: aString! !

!PUUser methodsFor: 'utilties' stamp: 'lr 3/9/2006 11:31'!
removeGroup: aGroup
	self groups remove: aGroup.
	aGroup users remove: self! !

!PUUser methodsFor: 'accessing' stamp: 'lr 3/9/2006 11:31'!
superuser
	^ superuser! !

!PUUser methodsFor: 'accessing' stamp: 'lr 3/9/2006 11:31'!
superuser: aBoolean
	superuser := aBoolean! !

!PUUser methodsFor: 'validating' stamp: 'lr 9/11/2008 16:20'!
validatePassword: aString
	^ aString isNil not and: [ aString isEmpty not and: [ (PRCompatibility hashPassword: aString) = password ] ]! !

PRObject subclass: #PUPermission
	instanceVariableNames: 'command'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pier-Security-Model'!

PUPermission subclass: #PUAddPermission
	instanceVariableNames: 'structure'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pier-Security-Model'!

!PUAddPermission methodsFor: 'comparing' stamp: 'lr 9/11/2006 10:17'!
= anObject
	^ super = anObject
		and: [ self structure = anObject structure ]! !

!PUAddPermission methodsFor: 'comparing' stamp: 'lr 9/11/2006 10:17'!
hash
	^ super hash bitXor: self structure hash! !

!PUAddPermission methodsFor: 'accessing' stamp: 'lr 9/11/2006 10:16'!
label
	^ super label , ' ' , self structure label! !

!PUAddPermission methodsFor: 'initialization' stamp: 'lr 9/11/2006 10:15'!
setStructure: aClass
	structure := aClass! !

!PUAddPermission methodsFor: 'accessing' stamp: 'lr 9/11/2006 10:16'!
structure
	^ structure! !

!PUPermission class methodsFor: 'accessing-description' stamp: 'lr 4/10/2008 14:17'!
descriptionLabel
	^ MAStringDescription new
		accessor: #label;
		yourself! !

!PUPermission class methodsFor: 'testing' stamp: 'lr 9/11/2006 10:10'!
isAbstract
	^ false! !

!PUPermission class methodsFor: 'instance-creation' stamp: 'lr 9/11/2006 10:15'!
on: aClass
	^ self new
		setCommand: aClass;
		yourself! !

!PUPermission methodsFor: 'comparing' stamp: 'lr 9/11/2006 10:10'!
= aCommand
	^ self species = aCommand species
		and: [ self command = aCommand command ]! !

!PUPermission methodsFor: 'accessing' stamp: 'lr 9/11/2006 10:09'!
command
	"Answer the command class this permission applies to."

	^ command! !

!PUPermission methodsFor: 'comparing' stamp: 'lr 9/11/2006 10:24'!
hash
	^ self command hash! !

!PUPermission methodsFor: 'accessing' stamp: 'lr 9/11/2006 10:13'!
label
	"Answer the label this command class is refered to in the user interface."

	^ self command label! !

!PUPermission methodsFor: 'printing' stamp: 'lr 9/11/2006 10:10'!
printOn: aStream
	aStream nextPutAll: self label! !

!PUPermission methodsFor: 'initialization' stamp: 'lr 9/11/2006 09:56'!
setCommand: aClass
	command := aClass! !

!PUPermission methodsFor: 'testing' stamp: 'lr 9/11/2006 10:59'!
validCommand: aCommand
	^ self command = aCommand species! !

!PRAddCommand class methodsFor: '*pier-security' stamp: 'lr 9/11/2006 10:21'!
permissions
	^ PRStructure withAllConcreteClasses
		collect: [ :each | (PUAddPermission on: self) setStructure: each ]! !

!PRAddCommand methodsFor: '*pier-security' stamp: 'lr 9/11/2006 11:13'!
permission
	^ (PUAddPermission on: self class) setStructure: self type! !

!PRWidget methodsFor: '*pier-security-testing' stamp: 'lr 9/27/2006 14:19'!
isSuperuser
	^ self context user notNil and: [ self context user isSuperuser ]! !

PRWidget subclass: #PULinksWidget
	instanceVariableNames: 'report'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pier-Security-Widgets'!

!PULinksWidget class methodsFor: 'accessing' stamp: 'lr 4/6/2009 13:00'!
groupLabel
	^ 'Pier System'! !

!PULinksWidget class methodsFor: 'testing' stamp: 'lr 4/6/2009 13:00'!
isAbstract
	^ false! !

!PULinksWidget class methodsFor: 'accessing' stamp: 'lr 4/6/2009 13:01'!
label
	^ 'Links'! !

!PULinksWidget methodsFor: 'private' stamp: 'lr 4/6/2009 20:56'!
buildReport
	| component |
	component := MAReport rows: self findLinks.
	component showFilter: true; batchSize: 25.
	component addColumn: (MAColumn new
		format: [ :value :html | self renderLink: value on: html ];
		accessor: #yourself;
		title: 'Link';
		yourself).
	component addColumn: (MAColumn new
		accessor: #(class label);
		title: 'Kind';
		filter: true;
		yourself).
	component addColumn: (MAColumn new
		format: [ :value :html | self renderOwner: value on: html ];
		accessor: #owner;
		title: 'Owner';
		filter: true;
		sorter: nil;
		yourself).
	component addColumn: (MAColumn new
		accessor: #isEmbedded;
		title: 'Embedded';
		filter: true;
		sorter: nil;
		yourself).
	component addColumn: (MAColumn new
		accessor: #isBroken;
		title: 'Broken';
		filter: true;
		sorter: nil;
		yourself).
	^ component! !

!PULinksWidget methodsFor: 'accessing' stamp: 'lr 4/6/2009 13:19'!
children
	^ Array with: self report! !

!PULinksWidget methodsFor: 'private' stamp: 'lr 4/6/2009 19:51'!
findLinks
	| links |
	links := OrderedCollection new: 1024.
	(self context enumeratorFor: self context root) everything do: [ :structure |
		structure outgoingReferences
			continue: [ :link | false ];
			do: [ :each | 
				each owner isNil 
					ifFalse: [ links addLast: each ] ] ].
	^ links! !

!PULinksWidget methodsFor: 'rendering' stamp: 'lr 4/6/2009 21:03'!
renderActionsOn: html
	html form: [
		html submitButton
			callback: [ report := nil ];
			with: 'Refresh'.
		html submitButton
			callback: [ 
				self session returnResponse: (WAResponse 
					document: self report export
					mimeType: 'application/excel'
					fileName: 'links.xls') ];
			with: 'Export' ]! !

!PULinksWidget methodsFor: 'rendering' stamp: 'lr 4/6/2009 13:42'!
renderContentOn: html
	self renderReportOn: html.
	self renderActionsOn: html! !

!PULinksWidget methodsFor: 'rendering-columns' stamp: 'lr 4/6/2009 13:58'!
renderLink: aLink on: html
	| document |
	(aLink isKindOf: PRValueLink)
		ifTrue: [ document := PRText content: aLink text ]
		ifFalse: [ 
			document := aLink copy.
			document setChildren: #(); parameters: #(); embedded: false.
			document add: (PRText content: aLink reference).
			document propertyAt: 'title' put: aLink text ].
	self context renderer
		continue: document
		in: self report 
		on: html! !

!PULinksWidget methodsFor: 'rendering-columns' stamp: 'lr 4/6/2009 14:48'!
renderOwner: aLink on: html
	html anchor
		goto: (self context structure: aLink owner);
		with: (PRPathReference to: aLink owner)! !

!PULinksWidget methodsFor: 'rendering' stamp: 'lr 4/6/2009 13:19'!
renderReportOn: html
	html render: self report! !

!PULinksWidget methodsFor: 'accessing' stamp: 'lr 4/6/2009 13:19'!
report
	^ report ifNil: [ report := self buildReport ]! !

PRWidget subclass: #PUMemberWidget
	instanceVariableNames: 'report'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pier-Security-Widgets'!

PUMemberWidget subclass: #PUGroupsWidget
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pier-Security-Widgets'!

!PUGroupsWidget class methodsFor: 'testing' stamp: 'lr 9/27/2006 10:30'!
isAbstract
	^ false! !

!PUGroupsWidget class methodsFor: 'accessing' stamp: 'lr 9/3/2008 11:50'!
label
	^ 'Groups'! !

!PUGroupsWidget methodsFor: 'accessing' stamp: 'lr 9/27/2006 10:52'!
memberClass
	^ PUGroup! !

!PUGroupsWidget methodsFor: 'accessing' stamp: 'lr 9/27/2006 10:54'!
members
	^ self context kernel groups! !

!PUGroupsWidget methodsFor: 'actions' stamp: 'lr 9/27/2006 11:32'!
remove: aMember
	aMember users isEmpty
		ifFalse: [ ^ self inform: 'Remove all users before removing a group.' ].
	super remove: aMember! !

!PUMemberWidget class methodsFor: 'accessing' stamp: 'lr 9/3/2008 11:46'!
groupLabel
	^ 'Pier System'! !

!PUMemberWidget methodsFor: 'actions' stamp: 'lr 2/12/2007 19:07'!
add: aString
	| member |
	aString isEmptyOrNil
		ifTrue: [ ^ self ].
	self members do: [ :each |
		each name = aString
			ifTrue: [ ^ self inform: 'A member with the name ' , aString printString , ' already exists.' ] ].
	member := self
		edit: (self memberClass named: aString)
		title: 'Add'.
	member ifNil: [ ^ self ].
	self members add: member.
	self refresh! !

!PUMemberWidget methodsFor: 'private' stamp: 'lr 9/26/2007 19:15'!
buildReport
	| component |
	component := MAReport
		rows: self members asArray
		description: self memberClass description.
	component addColumn: (MACommandColumn new
		visible: self isSuperuser;
		addCommandOn: self selector: #edit: text: 'edit';
		addCommandOn: self selector: #remove: text: 'remove';
		yourself).
	^ component! !

!PUMemberWidget methodsFor: 'accessing' stamp: 'lr 2/12/2007 19:07'!
children
	^ Array with: self report! !

!PUMemberWidget methodsFor: 'actions' stamp: 'lr 9/27/2006 11:13'!
edit: aMember
	| member |
	member := self edit: aMember title: 'Edit'.
	member ifNotNil: [ self refresh ].
	^ member! !

!PUMemberWidget methodsFor: 'actions' stamp: 'lr 9/26/2007 19:21'!
edit: aMember title: aString
	self isSuperuser 
		ifFalse: [ ^ nil ].
	^ self call: (aMember asComponent
		addMessage: aString;
		addValidatedForm;
		yourself)! !

!PUMemberWidget methodsFor: 'accessing' stamp: 'lr 10/30/2006 19:30'!
memberClass
	self subclassResponsibility! !

!PUMemberWidget methodsFor: 'accessing' stamp: 'lr 10/30/2006 19:29'!
members
	self subclassResponsibility! !

!PUMemberWidget methodsFor: 'events' stamp: 'lr 9/26/2007 19:15'!
onChangeContext: aContext
	super onChangeContext: aContext.
	self report columns last visible: self isSuperuser! !

!PUMemberWidget methodsFor: 'actions' stamp: 'lr 9/27/2006 11:01'!
refresh
	self report rows: self members asArray! !

!PUMemberWidget methodsFor: 'actions' stamp: 'lr 9/26/2007 19:21'!
remove: aMember
	self isSuperuser 
		ifFalse: [ ^ nil ].
	(self confirm: 'Are you sure to remove ' , aMember name printString , '?')
		ifFalse: [ ^ nil ].
	self members remove: aMember.
	self refresh.
	^ aMember! !

!PUMemberWidget methodsFor: 'rendering' stamp: 'lr 9/27/2006 12:12'!
renderAddOn: html
	self isSuperuser
		ifFalse: [ ^ self ].
	html break.
	html form: [
		html textInput
			callback: [ :value | self add: value ].
		html submitButton
			text: 'add' ]! !

!PUMemberWidget methodsFor: 'rendering' stamp: 'lr 4/23/2007 07:43'!
renderContentOn: html
	self renderReportOn: html.
	self renderAddOn: html! !

!PUMemberWidget methodsFor: 'rendering' stamp: 'lr 9/27/2006 10:59'!
renderReportOn: html
	html render: self report! !

!PUMemberWidget methodsFor: 'accessing' stamp: 'lr 9/27/2006 10:58'!
report
	^ report ifNil: [ report := self buildReport ]! !

PUMemberWidget subclass: #PUUsersWidget
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pier-Security-Widgets'!

!PUUsersWidget class methodsFor: 'testing' stamp: 'lr 9/27/2006 10:30'!
isAbstract
	^ false! !

!PUUsersWidget class methodsFor: 'accessing' stamp: 'lr 9/3/2008 11:50'!
label
	^ 'Users'! !

!PUUsersWidget methodsFor: 'accessing' stamp: 'lr 9/27/2006 10:52'!
memberClass
	^ PUUser! !

!PUUsersWidget methodsFor: 'accessing' stamp: 'lr 9/27/2006 10:54'!
members
	^ self context kernel users! !

!PUUsersWidget methodsFor: 'actions' stamp: 'lr 9/27/2006 11:16'!
remove: aMember
	(super remove: aMember)
		ifNotNil: [ aMember groups: #() ]! !

PRWidget subclass: #PUSecurityWidget
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pier-Security-Widgets'!

!PUSecurityWidget class methodsFor: 'accessing' stamp: 'lr 9/3/2008 11:46'!
groupLabel
	^ 'Pier System'! !

!PUSecurityWidget class methodsFor: 'testing' stamp: 'lr 9/27/2006 12:16'!
isAbstract
	^ false! !

!PUSecurityWidget class methodsFor: 'accessing' stamp: 'lr 9/3/2008 11:50'!
label
	^ 'Credentials'! !

!PUSecurityWidget methodsFor: 'actions' stamp: 'lr 2/12/2007 19:07'!
editUser
	"Ugly, but it works for now."

	| component |
	component := (self context user description
		select: [ :each | each hasProperty: #public ])
			asComponentOn: self context user.
	component
		addValidatedForm;
		addMessage: 'Edit User'.
	(self context components
		detect: [ :each | each isKindOf: PRContentsWidget ]
		ifNone: [ ^ self ])
			call: component! !

!PUSecurityWidget methodsFor: 'rendering' stamp: 'tg 2/1/2009 15:02'!
renderContentOn: html
	| user |
	user := self context user.
	user isNil ifFalse: [
		html span class: 'changePassword'; with: [html anchor callback: [ self editUser]; with: 'change password' ].
	].! !

PRWidget subclass: #PUSystemWidget
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pier-Security-Widgets'!

!PUSystemWidget class methodsFor: 'accessing' stamp: 'lr 9/3/2008 11:46'!
groupLabel
	^ 'Pier System'! !

!PUSystemWidget class methodsFor: 'testing' stamp: 'lr 9/27/2006 14:18'!
isAbstract
	^ false! !

!PUSystemWidget class methodsFor: 'accessing' stamp: 'lr 9/27/2006 14:18'!
label
	^ 'System'! !

!PUSystemWidget methodsFor: 'accessing' stamp: 'lr 2/3/2007 19:17'!
persistency
	^ self context kernel persistency! !

!PUSystemWidget methodsFor: 'rendering' stamp: 'lr 2/12/2007 19:07'!
renderActionsOn: html
	html form: [
		html submitButton
			callback: [ Smalltalk garbageCollect ];
			text: 'Garbage Collect'.
		(self persistency respondsTo: #snapshot) ifTrue: [
			html submitButton
				callback: [ self persistency snapshot ];
				text: 'Snapshot' ] ]! !

!PUSystemWidget methodsFor: 'rendering' stamp: 'lr 4/23/2007 07:43'!
renderContentOn: html
	html preformatted: SeasidePlatformSupport vmStatisticsReportString.
	self isSuperuser ifTrue: [ self renderActionsOn: html ]! !

!PRForbiddenView methodsFor: '*pier-security-rendering' stamp: 'lr 9/4/2008 13:37'!
renderOptionLoginOn: html
	<option: 0>
	
	| loginContext |
	loginContext := self context
		structure: self context structure
		command: (PULogin new
			answer: forbidden;
			yourself).
	loginContext isValid ifTrue: [
		html listItem: [
			html anchor
				goto: loginContext;
				with: 'Authenticate'.
			html text: ' yourself to the site.' ] ]! !

!PRKernelTest methodsFor: '*pier-security-testing' stamp: 'nfr 10/18/2007 11:51'!
testGroupNamed
	| adminGroup found |
	self
		assert: (self kernel groupNamed: 'admin' ifNone: [self fail]) isNil not
		description: 'Should find admin group by its name'.
	adminGroup := self kernel groupNamed: 'admin' ifNone: [].
	self assert: (adminGroup isKindOf: PUGroup).
	self assert: (adminGroup name = 'admin').
	found := true.
	self kernel groupNamed: 'a group that is not there'
		ifNone: [found := false].
	self deny: found description: 'Should not find a group that is not there'! !

!PRKernelTest methodsFor: '*pier-security-testing' stamp: 'nfr 10/18/2007 11:52'!
testUserNamed
	| adminUser found |
	self
		assert: (self kernel userNamed: 'admin' ifNone: [self fail]) isNil not
		description: 'Should find admin group by its name'.
	adminUser := self kernel userNamed: 'admin' ifNone: [].
	self assert: (adminUser isKindOf: PUUser).
	self assert: (adminUser name = 'admin').
	found := true.
	self kernel userNamed: 'a user that is not there' ifNone: [found := false].
	self deny: found description: 'Should not find a user that is not there'! !

!PRCommand class methodsFor: '*pier-security' stamp: 'lr 9/11/2006 10:21'!
permissions
	"Answer a collection of permissions as defined by the receiver."

	^ Array with: (PUPermission on: self)! !

!PRCommand methodsFor: '*pier-security' stamp: 'lr 11/6/2007 08:22'!
permission
	"The permission required to make the receiver executable in the setup context. ==nil== if no permission is required."

	| permissions |
	permissions := self class permissions.
	^ permissions isEmpty ifFalse: [ permissions anyOne ]! !

PRCommand subclass: #PUChangeSecurity
	instanceVariableNames: 'with all operator permissions'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pier-Security-Command'!

PUChangeSecurity subclass: #PUChangeGroup
	instanceVariableNames: 'group'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pier-Security-Command'!

!PUChangeGroup class methodsFor: 'accessing' stamp: 'lr 5/3/2006 16:58'!
accessKey
	^ $g! !

!PUChangeGroup class methodsFor: 'accessing-description' stamp: 'lr 4/10/2008 14:17'!
descriptionGroup
	^ MASingleOptionDescription new
		accessor: #group;
		label: 'Group';
		priority: 200;
		comment: 'Changes the group ownership to the selected group.';
		options: [ self groupInstances ] asDynamicObject;
		reference: PUGroup description;
		undefined: '(unchanged)';
		beSorted;
		yourself! !

!PUChangeGroup class methodsFor: 'testing' stamp: 'lr 3/9/2006 11:31'!
isAbstract
	^ false! !

!PUChangeGroup class methodsFor: 'accessing' stamp: 'lr 3/9/2006 11:31'!
label
	^ 'Change Group'! !

!PUChangeGroup methodsFor: 'configuration' stamp: 'lr 10/6/2008 11:42'!
accessor
	^ #(securityDecoration groupPermissions) asAccessor! !

!PUChangeGroup methodsFor: 'events' stamp: 'lr 3/9/2006 11:30'!
doExecuteFor: aStructure
	self group notNil
		ifTrue: [ aStructure securityDecoration group: self group ].
	super doExecuteFor: aStructure! !

!PUChangeGroup methodsFor: 'accessing' stamp: 'lr 3/9/2006 11:30'!
group
	^ group ifNil: [ group := self structure group ]! !

!PUChangeGroup methodsFor: 'accessing' stamp: 'lr 3/9/2006 11:30'!
group: aGroup
	group := aGroup! !

PUChangeSecurity subclass: #PUChangeOther
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pier-Security-Command'!

!PUChangeOther class methodsFor: 'accessing' stamp: 'lr 5/3/2006 16:58'!
accessKey
	^ $o! !

!PUChangeOther class methodsFor: 'testing' stamp: 'lr 3/9/2006 11:31'!
isAbstract
	^ false! !

!PUChangeOther class methodsFor: 'accessing' stamp: 'lr 3/9/2006 11:31'!
label
	^ 'Change Other'! !

!PUChangeOther methodsFor: 'configuration' stamp: 'lr 10/6/2008 11:42'!
accessor
	^ #(securityDecoration otherPermissions) asAccessor! !

PUChangeSecurity subclass: #PUChangeOwner
	instanceVariableNames: 'owner'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pier-Security-Command'!

!PUChangeOwner class methodsFor: 'accessing-description' stamp: 'lr 4/10/2008 14:17'!
descriptionOwner
	^ MASingleOptionDescription new
		accessor: #owner;
		label: 'Owner';
		priority: 200;
		comment: 'Changes the ownership to the selected user.';
		options: [ self userInstances ] asDynamicObject;
		reference: PUUser description;
		undefined: '(unchanged)';
		beSorted;
		yourself! !

!PUChangeOwner class methodsFor: 'testing' stamp: 'lr 3/9/2006 11:31'!
isAbstract
	^ false! !

!PUChangeOwner class methodsFor: 'accessing' stamp: 'lr 3/9/2006 11:31'!
label
	^ 'Change Owner'! !

!PUChangeOwner methodsFor: 'accessing' stamp: 'lr 10/6/2008 11:42'!
accessor
	^ #(securityDecoration ownerPermissions) asAccessor! !

!PUChangeOwner methodsFor: 'events' stamp: 'lr 3/9/2006 11:30'!
doExecuteFor: aStructure
	self owner notNil
		ifTrue: [ aStructure securityDecoration owner: self owner ].
	super doExecuteFor: aStructure! !

!PUChangeOwner methodsFor: 'accessing' stamp: 'lr 3/9/2006 11:30'!
owner
	^ owner ifNil: [ owner := self structure owner ]! !

!PUChangeOwner methodsFor: 'accessing' stamp: 'lr 3/9/2006 11:30'!
owner: anUser
	owner := anUser! !

!PUChangeSecurity class methodsFor: 'accessing-defaults' stamp: 'lr 1/12/2007 16:18'!
defaultAll
	^ false! !

!PUChangeSecurity class methodsFor: 'accessing-defaults' stamp: 'lr 3/9/2006 11:31'!
defaultOperator
	^ self defaultOperators first! !

!PUChangeSecurity class methodsFor: 'accessing-defaults' stamp: 'lr 3/9/2006 11:31'!
defaultOperators
	^ #( set add remove )! !

!PUChangeSecurity class methodsFor: 'accessing-defaults' stamp: 'lr 3/9/2006 11:31'!
defaultWith
	^ true! !

!PUChangeSecurity class methodsFor: 'accessing-description' stamp: 'lr 4/10/2008 14:17'!
descriptionAll
	^ MABooleanDescription new
		accessor: #all;
		label: 'Recursive';
		priority: 110;
		default: self defaultAll;
		comment: 'Apply changes recursively to all children.';
		beRequired;
		yourself! !

!PUChangeSecurity class methodsFor: 'accessing-description' stamp: 'lr 4/10/2008 14:17'!
descriptionOperator
	^ MASingleOptionDescription new
		accessor: #operator;
		label: 'Operator';
		priority: 300;
		default: self defaultOperator;
		comment: 'Defines if the permissions should remain unchanged, be set to the selected ones, or be added/removed from the current selection.';
		reference: MASymbolDescription new;
		options: self defaultOperators;
		undefined: '(unchanged)';
		yourself! !

!PUChangeSecurity class methodsFor: 'accessing-description' stamp: 'lr 4/10/2008 14:17'!
descriptionPermissions
	^ MAMultipleOptionDescription new
		accessor: #permissions;
		label: 'Permissions';
		priority: 310;
		comment: 'Defines the set of permissions used together with the selected operation.';
		options: [ self permissionInstances ] asDynamicObject;
		reference: PUPermission description;
		beSorted;
		yourself! !

!PUChangeSecurity class methodsFor: 'accessing-description' stamp: 'lr 4/10/2008 14:17'!
descriptionWith
	^ MABooleanDescription new
		accessor: #with;
		label: 'Current';
		priority: 100;
		default: self defaultWith;
		comment: 'Apply changes to the current structure.';
		beRequired;
		yourself! !

!PUChangeSecurity class methodsFor: 'private' stamp: 'lr 3/9/2006 11:31'!
groupInstances
	^ PRCurrentContext value kernel groups! !

!PUChangeSecurity class methodsFor: 'private' stamp: 'lr 11/5/2007 21:17'!
permissionInstances
	^ PRCommand withAllConcreteClasses gather: [ :each | each permissions ]! !

!PUChangeSecurity class methodsFor: 'private' stamp: 'lr 3/9/2006 11:31'!
userInstances
	^ PRCurrentContext value kernel users! !

!PUChangeSecurity methodsFor: 'accessing' stamp: 'lr 3/9/2006 11:30'!
accessor
	self subclassResponsibility! !

!PUChangeSecurity methodsFor: 'accessing-scope' stamp: 'lr 3/9/2006 11:30'!
all
	^ all ifNil: [ all := self class defaultAll ]! !

!PUChangeSecurity methodsFor: 'accessing-scope' stamp: 'lr 3/9/2006 11:30'!
all: aBoolean
	all := aBoolean! !

!PUChangeSecurity methodsFor: 'events' stamp: 'lr 9/27/2007 10:04'!
doExecute
	super doExecute.
	self structure enumerator
		with: self with; all: self all;
		do: [ :each |
			(self all or: [ self structure = each ])
				ifTrue: [ self doExecuteFor: each ] ]! !

!PUChangeSecurity methodsFor: 'events' stamp: 'lr 2/12/2007 19:07'!
doExecuteFor: aStructure
	"Apply the changes to aStructure. Override this method to further modify aStructure. This method is automatically called by #doExecute for all the requested structure."

	self accessor
		write: (self operator = #set
			ifTrue: [ self permissions copy ]
			ifFalse: [
				self operator = #add
					ifTrue: [
						(self accessor read: aStructure)
							copyWithAll: self permissions ]
					ifFalse: [
						self operator = #remove
							ifTrue: [
								(self accessor read: aStructure)
									copyWithoutAll: self permissions ]
							ifFalse: [ ^ self ] ] ])
		to: aStructure! !

!PUChangeSecurity methodsFor: 'accessing-operation' stamp: 'lr 3/9/2006 11:30'!
operator
	^ operator! !

!PUChangeSecurity methodsFor: 'accessing-operation' stamp: 'lr 3/9/2006 11:30'!
operator: aSymbol
	operator := aSymbol! !

!PUChangeSecurity methodsFor: 'accessing-operation' stamp: 'lr 3/9/2006 11:30'!
permissions
	^ permissions ifNil: [ permissions := self accessor read: self structure ]! !

!PUChangeSecurity methodsFor: 'accessing-operation' stamp: 'lr 3/9/2006 11:30'!
permissions: aCollectionOfCommands
	permissions := aCollectionOfCommands asSet! !

!PUChangeSecurity methodsFor: 'accessing' stamp: 'lr 8/11/2007 21:58'!
saveButton
	^ super saveButton value: 'Apply'! !

!PUChangeSecurity methodsFor: 'accessing-scope' stamp: 'lr 3/9/2006 11:30'!
with
	^ with ifNil: [ with := self class defaultWith ]! !

!PUChangeSecurity methodsFor: 'accessing-scope' stamp: 'lr 3/9/2006 11:30'!
with: aBoolean
	with := aBoolean! !

PRCommand subclass: #PUChangeUser
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pier-Security-Command'!

!PUChangeUser class methodsFor: 'accessing' stamp: 'lr 11/5/2007 21:15'!
accessKey
	^ $l! !

!PUChangeUser class methodsFor: 'accessing' stamp: 'lr 11/5/2007 21:15'!
permissions
	^ #()! !

!PUChangeUser methodsFor: 'events' stamp: 'lr 2/20/2009 00:33'!
doValidate
	"Change user commands do not require validation."! !

!PUChangeUser methodsFor: 'testing' stamp: 'lr 2/20/2009 00:28'!
isLogged
	^ false! !

PUChangeUser subclass: #PULogin
	instanceVariableNames: 'ancestor username password user'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pier-Security-Command'!

!PULogin class methodsFor: 'accessing-description' stamp: 'lr 4/10/2008 14:17'!
descriptionPassword
	^ MAPasswordDescription new
		accessor: #password;
		label: 'Password';
		priority: 200;
		beRequired;
		yourself! !

!PULogin class methodsFor: 'accessing-description' stamp: 'lr 4/10/2008 14:17'!
descriptionUsername
	^ MAStringDescription new
		accessor: #username;
		label: 'Username';
		priority: 100;
		beRequired;
		yourself! !

!PULogin class methodsFor: 'testing' stamp: 'lr 3/9/2006 11:31'!
isAbstract
	^ false! !

!PULogin class methodsFor: 'testing' stamp: 'lr 3/9/2006 11:31'!
isValidIn: aContext
	^ aContext isLoggedIn not! !

!PULogin class methodsFor: 'accessing' stamp: 'lr 3/9/2006 11:31'!
label
	^ 'Login'! !

!PULogin methodsFor: 'events' stamp: 'lr 10/22/2007 22:16'!
doAnswer
	super doAnswer.
	self answer: (self answer user: self user)! !

!PULogin methodsFor: 'events' stamp: 'lr 2/3/2007 19:23'!
doValidate
	super doValidate.
	self user: (self kernel users
		detect: [ :each |
			each name = self username
				and: [ each validatePassword: self password ] ]
		ifNone: [ self invalidLogin ])! !

!PULogin methodsFor: 'private' stamp: 'lr 2/3/2007 19:25'!
invalidLogin
	PUInvalidLogin
		description: self description
		signal: 'Invalid username or password.'! !

!PULogin methodsFor: 'configuration' stamp: 'lr 2/3/2007 19:25'!
mementoClass
	^ MACachedMemento! !

!PULogin methodsFor: 'accessing' stamp: 'lr 3/9/2006 11:30'!
password
	^ password ifNil: [ password := String new ]! !

!PULogin methodsFor: 'accessing' stamp: 'lr 3/9/2006 11:30'!
password: aString
	password := aString! !

!PULogin methodsFor: 'configuration' stamp: 'lr 8/11/2007 21:58'!
saveButton
	^ super saveButton value: self class label! !

!PULogin methodsFor: 'accessing' stamp: 'lr 3/9/2006 11:30'!
user
	^ user! !

!PULogin methodsFor: 'accessing' stamp: 'lr 3/9/2006 11:30'!
user: aUser
	user := aUser! !

!PULogin methodsFor: 'accessing' stamp: 'lr 3/9/2006 11:30'!
username
	^ username ifNil: [ username := String new ]! !

!PULogin methodsFor: 'accessing' stamp: 'lr 3/9/2006 11:30'!
username: aString
	username := aString! !

PUChangeUser subclass: #PULogout
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pier-Security-Command'!

!PULogout class methodsFor: 'testing' stamp: 'lr 3/9/2006 11:31'!
isAbstract
	^ false! !

!PULogout class methodsFor: 'testing' stamp: 'lr 3/9/2006 11:31'!
isValidIn: aContext
	^ aContext isLoggedIn! !

!PULogout class methodsFor: 'accessing' stamp: 'lr 3/9/2006 11:31'!
label
	^ 'Logout'! !

!PULogout methodsFor: 'events' stamp: 'lr 3/21/2006 18:08'!
doAnswer
	super doAnswer.
	self answer: (self answer user: nil)! !

!PULogout methodsFor: 'testing' stamp: 'lr 3/9/2006 11:31'!
isQuick
	^ true! !

!PRValueLink methodsFor: '*pier-security' stamp: 'lr 4/1/2009 21:46'!
groupIn: aContext
	<value: 'group' comment: 'Display the group of the current structure.'>
	
	^ self lookupStructure: aContext structure do: [ :structure |
		self formatDescribed: structure group default: #name ]! !

!PRValueLink methodsFor: '*pier-security' stamp: 'lr 4/1/2009 21:46'!
ownerIn: aContext
	<value: 'owner' comment: 'Display the owner of the current structure.'>

	^ self lookupStructure: aContext structure do: [ :structure |
		self formatDescribed: structure owner default: #name ]! !

!PRValueLink methodsFor: '*pier-security' stamp: 'lr 4/1/2009 21:46'!
userIn: aContext
	<value: 'user' comment: 'Display the currently authenticated structure.'>

	^ self formatDescribed: aContext user default: #name! !
